from typing import Union, Literal, Generic, TypeVar

from pydantic import BaseModel, Field
from pydantic.generics import GenericModel

DataT = TypeVar('DataT')


class FastAPIErrorDetail(BaseModel):
    loc: list[str]
    message: str = Field(alias='msg')
    type: str


class FastApiErrorResponse(BaseModel):
    detail: FastAPIErrorDetail | str


class APIErrorResponse(BaseModel):
    error_code: int
    message: str


class Response(GenericModel):
    data: DataT | None
    error: APIErrorResponse | FastApiErrorResponse | None
    success: bool = True


class MembersAPIPlayer(BaseModel):
    username: str
    id: int | None
    is_online: bool = Field(alias='isOnline')
    is_leader: bool = Field(alias='isLeader')
    rank: int
    rank_label: str | None = Field(alias='rankLabel')


class MembersAPIResponse(BaseModel):
    server: str
    fraction_label: str = Field(alias='fractionLabel')
    players: list[MembersAPIPlayer]
    total_players: int = Field(alias='totalPlayers')
    total_online: int = Field(alias='totalOnline')
    leader_nickname: str = Field(alias='leaderNick')
    is_leader_online: bool = Field('isLeaderOnline')


class ServerStatusAPIResponse(BaseModel):
    hostname: str
    payday_multiplier: int = Field(alias='paydayMultiplier')
    online_players: int = Field(alias='onlinePlayers')
    max_players: int = Field(alias='maxPlayers')
    is_closed: bool = Field(alias='isClosed')
    ip: str
    server_label: str = Field(alias='serverLabel')


class CreatedFindTaskAPIResponse(BaseModel):
    request_id: str
    request_time: int


class PlayerInfoArizonaAPIResponse(BaseModel):
    account_id: int = Field(alias='accountId')
    player_id: int | None = Field(alias='playerId')
    lvl: int
    cash: int
    bank: int
    individual_account: int | None = Field(alias='individualAccount')
    deposit: int
    total_money: int = Field(alias='totalMoney')
    is_online: bool = Field(alias='isOnline')
    job_label: str = Field(alias='jobLabel')
    job_id: int = Field(alias='jobId')
    rank_number: int | None = Field(alias='rankNumber')
    rank_label: str | None = Field(alias='rankLabel')
    is_leader: bool = Field(alias='isLeader')
    org_label: str = Field(alias='orgLabel')
    org_id: int = Field(alias='orgId')
    vip_lvl: int = Field(alias='vipLvl')
    vip_label: str = Field(alias='vipLabel')
    phone_number: int | None = Field(alias='phoneNumber')
    updated_at: int = Field(alias='updatedAt')
    is_new_request: bool = Field(alias='newRequest')
    player_nick: str = Field(alias='playerNick')
    player_server: int = Field(alias='playerServer')
    server_name: str = Field(alias='serverName')


class PlayerInfoRodinaAPIResponse(BaseModel):
    hp: int
    hunger: int
    lvl: int
    vip: str
    cash: int
    bank: int
    azCoins: int
    isLeader: bool
    fraction: str
    rank: int
    job: str
    isOnline: bool


class RatingPlayerAPI(BaseModel):
    number: int
    name: str
    is_online: bool = Field('isOnline')


class RatingFamilyAPI(BaseModel):
    number: int
    name: str
    owner: str
    lvl: int


class RatingAPIResponse(BaseModel):
    server: str
    players: list[RatingPlayerAPI] | None
    families: list[RatingFamilyAPI] | None


class IpAPIResponse(BaseModel):
    ip: str
    country: str
    city: str
    timezone: str
    isp: str


class CheckRPUsernameName(BaseModel):
    rp: bool
    schedule: str | None


class CheckRPUsernameAPIResponse(BaseModel):
    name: CheckRPUsernameName
    surname: CheckRPUsernameName
    nickname: str = Field(alias='nick')


class GenerateRPUsernameAPIResponse(BaseModel):
    name: str
    surname: str




from .model import IpAPIResponse, MembersAPIResponse, PlayerInfoRodinaAPIResponse, PlayerInfoArizonaAPIResponse, \
    CreatedFindTaskAPIResponse, \
    ServerStatusAPIResponse, RatingAPIResponse, CheckRPUsernameAPIResponse, GenerateRPUsernameAPIResponse

from .api import get, post

from typing import Literal
import asyncio

from pydantic import parse_obj_as, ValidationError


class VprikolAPI:
    def __init__(self, token: str, base_url: str = 'https://api.vprikol.dev/'):
        if len(token) < 1:
            raise Exception('Токен передан неправильно.')
        self.headers = {'Authorization': f'Bearer {token}'}
        self.base_url = base_url

    async def get_ip_info(self, ip: str) -> IpAPIResponse:
        result = await get(url=f'{self.base_url}ip', headers=self.headers, params={'ip': ip})

        if not result.success:
            raise Exception(result.error)

        return IpAPIResponse(**result.data)

    async def get_members(self, server_id: int, fraction_id: int) -> MembersAPIResponse:
        result = await get(url=f'{self.base_url}members', headers=self.headers, params={'server': server_id,
                                                                                          'fraction_id': fraction_id})
        if not result.success:
            raise Exception(result.error)

        players = []
        for player in result.data['players']:
            players.append({'username': player, 'id': result.data['players'][player]['id'],
                            'isOnline': result.data['players'][player]['isOnline'],
                            'isLeader': result.data['players'][player]['isLeader'],
                            'rank': result.data['players'][player]['rank'],
                            'rankLabel': result.data['players'][player]['rankLabel']})
        result.data['players'] = players
        return MembersAPIResponse(**result.data)

    async def get_player_information(self, server_id: int, nickname: str) -> PlayerInfoRodinaAPIResponse \
                                                                             | PlayerInfoArizonaAPIResponse:
        task = await post(url=f'{self.base_url}find/createTask', headers=self.headers,
                          params={'server': server_id, 'nick': nickname})
        if not task.success:
            raise Exception(task.error)

        task = CreatedFindTaskAPIResponse(**task.data)
        while True:
            await asyncio.sleep(1)
            result = await get(url=f'{self.base_url}find/getTaskResult', headers=self.headers,
                               params={'request_id': task.request_id})

            if not result.success and result.error.error_code and result.error.error_code == 425:
                continue

            if result.error.error_code == 422:
                return result.error

            try:
                return PlayerInfoArizonaAPIResponse(**result.data)
            except ValidationError:
                return PlayerInfoRodinaAPIResponse(**result.data)

    async def get_server_status(self, server_id: int | None = None) -> ServerStatusAPIResponse | \
                                                                       list[ServerStatusAPIResponse]:
        if server_id:
            params = {'server': server_id}
        else:
            params = None

        result = await get(url=f'{self.base_url}status', headers=self.headers,
                           params=params)

        if not result.success:
            raise Exception(result.error)

        if isinstance(result.data, list):
            return parse_obj_as(list[ServerStatusAPIResponse], result.data)

    async def get_rating(self, server_id: int, first_type: Literal[1, 2, 3],
                         second_type: Literal[1, 2, 3] | None = None) -> RatingAPIResponse:
        if second_type:
            params = {'type': first_type, 'subtype': second_type, 'server': server_id}
        else:
            params = {'type': first_type, 'server': server_id}

        result = await get(url=f'{self.base_url}rating', headers=self.headers,
                           params=params)

        if not result.success:
            raise Exception(result.error)

        return RatingAPIResponse(**result.data)

    async def check_rp_nickname(self, nickname: str) -> CheckRPUsernameAPIResponse:
        result = await get(url=f'{self.base_url}checkrp', headers=self.headers,
                           params={'nick': nickname})

        if not result.success:
            raise Exception(result.error)

        return CheckRPUsernameAPIResponse(**result.data)

    async def generate_rp_nickname(self, gender: Literal['male', 'female'], nation: Literal[
        'russian', 'american', 'german', 'french', 'italian', 'japanese', 'latinos', 'swedish', 'danish', 'romanian']) \
            -> GenerateRPUsernameAPIResponse:
        result = await get(url=f'{self.base_url}rpnick', headers=self.headers,
                           params={'gender': gender, 'nation': nation})
        if not result.success:
            raise Exception(result.error)

        return GenerateRPUsernameAPIResponse(**result.data)



import aiohttp
import json
from .model import Response


async def get(url: str, params: dict | None = None, headers: dict | None = None) -> Response:
    async with aiohttp.ClientSession(headers=headers) as session:
        async with session.get(url, params=params) as response:
            response_json = json.loads(await response.text())
            if response.ok:
                return Response(data=response_json)
            return Response(error=response_json, success=False)


async def post(url: str, params: dict | None = None, headers: dict | None = None) -> Response:
    async with aiohttp.ClientSession(headers=headers) as session:
        async with session.post(url, params=params) as response:
            response_json = json.loads(await response.text())
            if response.ok:
                return Response(data=response_json)
            return Response(error=response_json, success=False)
